#!/bin/bash

# =============================================================================
# @author: <sylvain.herledan@oceandatlab.com>
# @date: 2016-11-16
#
# Build functions to generate the static libraries required to build
# the SEAScope client.
# =============================================================================

set -o nounset
set -o errexit
set -o pipefail

build_zlib()
{
  readonly ZLIB_VERSION='1.3.1';

	local root_dir;
	local src_dir;
	local build_dir;
	local rootfs;
	local uri;

  root_dir="${1}"; shift;
  src_dir="${root_dir}/sources";
  build_dir="${root_dir}/build";
  rootfs="${root_dir}/rootfs";
  uri="http://zlib.net/zlib-${ZLIB_VERSION}.tar.gz";

	# Download source if needed
	if [ ! -f "${src_dir}/zlib-${ZLIB_VERSION}.tar.gz" ];
	then
		wget "${uri}" -O "${src_dir}/zlib-${ZLIB_VERSION}.tar.gz"
	fi

	# Extract archive
	if [ ! -d "${build_dir}/zlib-${ZLIB_VERSION}" ];
	then
		tar xzf "${src_dir}/zlib-${ZLIB_VERSION}.tar.gz" -C "${build_dir}"
	fi

	# Build
	if [ ! -f "${rootfs}/lib/libz.a" ];
	then
		cd "${build_dir}/zlib-${ZLIB_VERSION}"
		./configure --prefix="${rootfs}" \
                --libdir="${rootfs}/lib" \
					      --static \
					      --64
		make -j "${CPUs}"
		make install
	fi

	if $MACOSX_UNIVERSAL && ! lipo "${rootfs}/lib/libz.a" -verify_arch arm64;
	then
		build_dir="${root_dir}/build_arm";
		rootfs="${root_dir}/rootfs_arm";

		# Extract archive
		if [ ! -d "${build_dir}/zlib-${ZLIB_VERSION}" ];
		then
			tar xzf "${src_dir}/zlib-${ZLIB_VERSION}.tar.gz" -C "${build_dir}"
		fi

		# Build
		if [ ! -f "${rootfs}/lib/libz.a" ];
		then
			cd "${build_dir}/zlib-${ZLIB_VERSION}"
			CFLAGS="${CFLAGS} -target arm64-apple-macos11" \
			CPPFLAGS="${CPPFLAGS} -target arm64-apple-macos11" \
			LDFLAGS="${LDFLAGS} -target arm64-apple-macos11" \
			./configure --prefix="${rootfs}" \
						--libdir="${rootfs}/lib" \
						--static \
						--64
			make -j "${CPUs}"
			make install
		fi

		# Create a universal (fat) version of the library that combines both archs
		lipo -create -output "${root_dir}/rootfs/lib/libz.a" \
							 "${root_dir}/rootfs/lib/libz.a" \
							 "${root_dir}/rootfs_arm/lib/libz.a"
	fi

	# Back to root
	cd "${root_dir}"
}

build_sqlite3()
{
	local root_dir;
	local src_dir;
	local build_dir;
	local rootfs;
	local uri;

  root_dir="${1}"; shift;
  src_dir="${root_dir}/sources";
  build_dir="${root_dir}/build";
  rootfs="${root_dir}/rootfs";
  uri="https://sqlite.org/2016/sqlite-src-3110000.zip";

	# Download source if needed
	if [ ! -f "${src_dir}/sqlite-src-3110000.zip" ];
	then
		wget "${uri}" -O "${src_dir}/sqlite-src-3110000.zip"
	fi

	# Extract archive
	if [ ! -d "${build_dir}/sqlite-src-3110000" ];
	then
		unzip "${src_dir}/sqlite-src-3110000.zip" -d "${build_dir}"
	fi

	# Build
	if [ ! -f "${rootfs}/lib/libsqlite3.a" ];
	then
		cd "${build_dir}/sqlite-src-3110000"

        set +o nounset
        local flags="${CPPFLAGS} -DSQLITE_ENABLE_COLUMN_METADATA=1 \
								 -DSQLITE_ENABLE_UNLOCK_NOTIFY \
								 -DSQLITE_ENABLE_DBSTAT_VTAB=1 \
								 -DSQLITE_SECURE_DELETE"
        set -o nounset

	    rm -f sqlite3.h
        CPPFLAGS="${flags}" \
        ./configure --prefix="${rootfs}" \
                    --libdir="${rootfs}/lib" \
                    --disable-amalgamation \
                    --disable-tcl \
                    --disable-readline \
                    --enable-fts3 \
                    --enable-fts4 \
                    --enable-fts5 \
                    --enable-rtree \
                    --enable-json1
		make # json extension breaks parallel build
		make install
	fi

	if $MACOSX_UNIVERSAL && ! lipo "${rootfs}/lib/libsqlite3.a" -verify_arch arm64;
	then
		build_dir="${root_dir}/build_arm";
		rootfs="${root_dir}/rootfs_arm";

		# Extract archive
		if [ ! -d "${build_dir}/sqlite-src-3110000" ];
		then
			unzip "${src_dir}/sqlite-src-3110000.zip" -d "${build_dir}"
		fi

		# Build
		if [ ! -f "${rootfs}/lib/libsqlite3.a" ];
		then
			cd "${build_dir}/sqlite-src-3110000"

			set +o nounset
			local flags="${CPPFLAGS} -DSQLITE_ENABLE_COLUMN_METADATA=1 \
									 -DSQLITE_ENABLE_UNLOCK_NOTIFY \
									 -DSQLITE_ENABLE_DBSTAT_VTAB=1 \
									 -DSQLITE_SECURE_DELETE"
			set -o nounset

			rm -f sqlite3.h
			CFLAGS="${CFLAGS} -target arm64-apple-macos11" \
			CPPFLAGS="${flags} -target arm64-apple-macos11" \
			LDFLAGS="${LDFLAGS} -target arm64-apple-macos11" \
			./configure --prefix="${rootfs}" \
						--libdir="${rootfs}/lib" \
						--disable-amalgamation \
						--disable-tcl \
						--disable-readline \
						--enable-fts3 \
						--enable-fts4 \
						--enable-fts5 \
						--enable-rtree \
						--enable-json1 \
						--host=arm
			make # json extension breaks parallel build
			make install
		fi

		# Create a universal (fat) version of the library that combines both archs
		lipo -create -output "${root_dir}/rootfs/lib/libsqlite3.a" \
							 "${root_dir}/rootfs/lib/libsqlite3.a" \
							 "${root_dir}/rootfs_arm/lib/libsqlite3.a"
	fi

	# Back to root
	cd "${root_dir}"
}

build_glfw()
{
	local root_dir;
	local src_dir;
	local build_dir;
	local rootfs;
	local uri;

  root_dir="${1}"; shift;
  src_dir="${root_dir}/sources";
  build_dir="${root_dir}/build";
  rootfs="${root_dir}/rootfs";
  uri="https://github.com/glfw/glfw.git";

	# Download source if needed
	if [ ! -d "${src_dir}/glfw" ];
	then
		git clone "${uri}" "${src_dir}/glfw"

		cd "${src_dir}/glfw"
		git checkout c32dc3a0856e339d8de582582fcbedec430e14b4
	fi

	# Build
	if [ ! -f "${rootfs}/lib/libglfw3.a" ];
	then
		mkdir -p "${build_dir}/glfw"
		cd "${build_dir}/glfw"
		cmake -DCMAKE_INSTALL_PREFIX="${rootfs}" \
			  -DCMAKE_INSTALL_LIBDIR="${rootfs}/lib" \
			  -DBUILD_SHARED_LIBS=OFF \
			  "${src_dir}/glfw"
		make -j "${CPUs}"
		make install
	fi

	if $MACOSX_UNIVERSAL && ! lipo "${rootfs}/lib/libglfw3.a" -verify_arch arm64;
	then
		build_dir="${root_dir}/build_arm";
		rootfs="${root_dir}/rootfs_arm";

		# Build
		if [ ! -f "${rootfs}/lib/libglfw3.a" ];
		then
			mkdir -p "${build_dir}/glfw"
			cd "${build_dir}/glfw"
			CFLAGS="${CFLAGS} -target arm64-apple-macos11" \
			CPPFLAGS="${CPPFLAGS} -target arm64-apple-macos11" \
			LDFLAGS="${LDFLAGS} -target arm64-apple-macos11" \
			cmake -DCMAKE_INSTALL_PREFIX="${rootfs}" \
				  -DCMAKE_INSTALL_LIBDIR="${rootfs}/lib" \
				  -DBUILD_SHARED_LIBS=OFF \
				  "${src_dir}/glfw"
			make -j "${CPUs}"
			make install
		fi

		# Create a universal (fat) version of the library that combines both archs
		lipo -create -output "${root_dir}/rootfs/lib/libglfw3.a" \
							 "${root_dir}/rootfs/lib/libglfw3.a" \
							 "${root_dir}/rootfs_arm/lib/libglfw3.a"
	fi

	# Back to root
	cd "${root_dir}"
}

build_ossp_uuid()
{
	local root_dir;
	local src_dir;
	local build_dir;
	local rootfs;
	local uri;

  root_dir="${1}"; shift;
  src_dir="${root_dir}/sources";
  build_dir="${root_dir}/build";
  rootfs="${root_dir}/rootfs";
  uri="https://mirrors.ocf.berkeley.edu/debian/pool/main/o/ossp-uuid/ossp-uuid_1.6.2.orig.tar.gz";

	# Download source if needed
	if [ ! -f "${src_dir}/uuid-1.6.2.tar.gz" ];
	then
		wget "${uri}" -O "${src_dir}/uuid-1.6.2.tar.gz"
	fi

	# Extract archive
	if [ ! -d "${build_dir}/uuid-1.6.2" ];
	then
		tar xzf "${src_dir}/uuid-1.6.2.tar.gz" -C "${build_dir}"
	fi

	# Build
	if [ ! -f "${rootfs}/lib/libuuid.a" ];
	then
		cd "${build_dir}/uuid-1.6.2"
		./configure --prefix="${rootfs}" \
					--libdir="${rootfs}/lib" \
					--without-perl \
					--without-php \
					--without-pgsql
		make -j "${CPUs}"
		make install
	fi

	if $MACOSX_UNIVERSAL && ! lipo "${rootfs}/lib/libuuid.a" -verify_arch arm64;
	then
		build_dir="${root_dir}/build_arm";
		rootfs="${root_dir}/rootfs_arm";

		# Extract archive
		if [ ! -d "${build_dir}/uuid-1.6.2" ];
		then
			tar xzf "${src_dir}/uuid-1.6.2.tar.gz" -C "${build_dir}"
		fi

		# Build
		if [ ! -f "${rootfs}/lib/libuuid.a" ];
		then
			cd "${build_dir}/uuid-1.6.2"

			# There's no easy way to cross-compile for arm64 so we fake it until we
			# make it

			# ./configure as if compiling for x86_64
			CFLAGS="${CFLAGS} -target x86_64-apple-macos${MACOSX_DEPLOYMENT_TARGET}" \
			CPPFLAGS="${CPPFLAGS} -target x86_64-apple-macos${MACOSX_DEPLOYMENT_TARGET}" \
			LDFLAGS="${LDFLAGS} -target x86_64-apple-macos${MACOSX_DEPLOYMENT_TARGET}" \
			./configure --prefix="${rootfs}" \
						--libdir="${rootfs}/lib" \
						--without-perl \
						--without-php \
						--without-pgsql

			# patch the -target flag
			sed -i '' \
				-e "s/-target x86_64-apple-macos${MACOSX_DEPLOYMENT_TARGET}/-target arm64-apple-macos11/g" \
				./Makefile
			sed -i '' \
				-e "s/-target x86_64-apple-macos${MACOSX_DEPLOYMENT_TARGET}/-target arm64-apple-macos11/g" \
				./libtool

			# patch libtool to disable building shared libraries (which will fail)
			sed -i '' \
				-e 's/build_libtool_libs=yes/build_libtool_libs=no/g' \
				./libtool

			# Finally we continue the compilation as normal
			make -j "${CPUs}"
			make install
		fi

		# Create a universal (fat) version of the library that combines both archs
		lipo -create -output "${root_dir}/rootfs/lib/libuuid.a" \
							 "${root_dir}/rootfs/lib/libuuid.a" \
							 "${root_dir}/rootfs_arm/lib/libuuid.a"
	fi

	# Back to root
	cd "${root_dir}"
}


build_abseil()
{
	local root_dir;
	local src_dir;
	local build_dir;
	local rootfs;
	local uri;

  root_dir="${1}"; shift;
  src_dir="${root_dir}/sources";
  build_dir="${root_dir}/build";
  rootfs="${root_dir}/rootfs";
  uri="https://github.com/abseil/abseil-cpp/archive/20230125.3/abseil-cpp-20230125.3.tar.gz";

	# Download source if needed
	if [ ! -f "${src_dir}/abseil-cpp-20230125.3.tar.gz" ];
	then
		wget "${uri}" -O "${src_dir}/abseil-cpp-20230125.3.tar.gz"
	fi

	# Extract archive
	if [ ! -d "${build_dir}/abseil-cpp-20230125.3" ];
	then
		tar xzf "${src_dir}/abseil-cpp-20230125.3.tar.gz" -C "${build_dir}"
	fi

	# Build
	if [ ! -f "${rootfs}/lib/libabsl_base.a" ];
	then
		cd "${build_dir}/abseil-cpp-20230125.3"
    mkdir build
    cmake -Bbuild \
          -DCMAKE_BUILD_TYPE=Release \
          -DCMAKE_CXX_FLAGS="${CXXFLAGS:-} ${CXX_FLAGS:-} -DNDEBUG" \
          -DCMAKE_INSTALL_PREFIX="${rootfs}" \
          -DCMAKE_INSTALL_LIBDIR="${rootfs}/lib" \
          -DCMAKE_CXX_STANDARD=14 \
          -DBUILD_SHARED_LIBS=OFF \
          -DBUILD_TESTING=OFF \
          -DABSL_PROPAGATE_CXX_STD=ON
    cd build
    cmake --build .
    cmake --install .
	fi
}
